package utils;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Date Utility Class This is used to convert Strings to Dates and Timestamps
 * 
 * <p>
 * <a href="DateUtil.java.html"><i>View Source</i></a>
 * </p>
 * 
 * @author <a href="mailto:matt@raibledesigns.com">Matt Raible</a> Modified by
 *         <a href="mailto:dan@getrolling.com">Dan Kibler </a> to correct time
 *         pattern. Minutes should be mm not MM (MM is month). <br> Modified by
 *         <a href="mailto:francois.pfister@ema.fr">Francois Pfister </a> french formats        
 *         
 * @version $Revision: 1.5 $ $Date: 2004/10/05 07:20:13 $
 */
public class DateUtils {
	// ~ Static fields/initializers
	// =============================================

	private static Log log = LogFactory.getLog(DateUtils.class);
	private static String outDatePattern = "dd/MM/yyyy";
	private static String timePattern = "HH:mm";
	private static DateFormat englishPattern1 = new SimpleDateFormat("EEE, dd MMM yyyy HH:MM:SS Z", Locale.ENGLISH); //Tue, 21 Mar 2006 12:12:20 EST
	private static DateFormat englishPattern2 = new SimpleDateFormat("EEE MMM dd HH:mm:ss Z yyyy", Locale.ENGLISH);  //Sat Jan 21 14:00:00 CET 2006
	private static DateFormat formatDMY = new SimpleDateFormat(outDatePattern);
	private static DateFormat formatYMD = new SimpleDateFormat("yyyy-MM-dd");
	private static DateFormat formatYMDHMS1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
	private static DateFormat formatYMDHMS2 = new SimpleDateFormat("yyyyMMddHHmmss", Locale.ENGLISH);

	// ~ Methods
	// ================================================================

	/**
	 * Return default datePattern (MM/dd/yyyy)
	 * 
	 * @return a string representing the date pattern on the UI
	 * 
	 * @uml.property name="datePattern"
	 */
	public static String getDatePattern() {
		return outDatePattern;
	}

	
	
	public static final String getYmdDate(Date aDate) {
		String returnValue = "";
		if (aDate != null) 
			returnValue = formatYMD.format(aDate);
		return (returnValue);
	}
	
	/**
	 * This method attempts to convert an Oracle-formatted date in the form
	 * dd-MMM-yyyy to mm/dd/yyyy.
	 * 
	 * @param aDate
	 *            date from database as a string
	 * @return formatted string for the ui
	 */
	public static final String getDate(Date aDate) {
		SimpleDateFormat df = null;
		String returnValue = "";

		if (aDate != null) {
			df = new SimpleDateFormat(outDatePattern);
			returnValue = df.format(aDate);
		}

		return (returnValue);
	}

	/**
	 * This method generates a string representation of a date/time in the
	 * format you specify on input
	 * 
	 * @param aMask
	 *            the date pattern the string is in
	 * @param strDate
	 *            a string representation of a date
	 * @return a converted Date object
	 * @see java.text.SimpleDateFormat
	 * @throws ParseException
	 */
	
	//MM/dd/yyyy HH:MM
	//2005-09-21 14:13:54
	//yyyy-MM-dd HH:MM:SS  "YYYY-MM-DD hh:mm:ss"
	public static final Date convertStringToDate(String aMask, String strDate)
			throws ParseException {
		SimpleDateFormat df = null;
		Date date = null;
		df = new SimpleDateFormat(aMask);

		if (log.isDebugEnabled()) {
			log.debug("converting '" + strDate + "' to date with mask '"
					+ aMask + "'");
		}

		try {
			date = df.parse(strDate);
		} catch (ParseException pe) {
			// log.error("ParseException: " + pe);
			throw new ParseException(pe.getMessage(), pe.getErrorOffset());
		}

		return (date);
	}

	/**
	 * This method returns the current date time in the format: MM/dd/yyyy HH:MM
	 * a
	 * 
	 * @param theTime
	 *            the current time
	 * @return the current date/time
	 */
	public static String getTimeNow(Date theTime) {
		return getDateTime(timePattern, theTime);
	}

	/**
	 * This method returns the current date in the format: MM/dd/yyyy
	 * 
	 * @return the current date
	 * @throws ParseException
	 */
	public static Calendar getToday() throws ParseException {
		Date today = new Date();
		SimpleDateFormat df = new SimpleDateFormat(outDatePattern);

		// This seems like quite a hack (date -> string -> date),
		// but it works ;-)
		String todayAsString = df.format(today);
		Calendar cal = new GregorianCalendar();
		cal.setTime(convertStringToDate(todayAsString));

		return cal;
	}

	/**
	 * This method generates a string representation of a date's date/time in
	 * the format you specify on input
	 * 
	 * @param aMask
	 *            the date pattern the string is in
	 * @param aDate
	 *            a date object
	 * @return a formatted string representation of the date
	 * 
	 * @see java.text.SimpleDateFormat
	 */
	public static final String getDateTime(String aMask, Date aDate) {
		SimpleDateFormat df = null;
		String returnValue = "";

		if (aDate == null) {
			//log.error("aDate is null!");
		} else {
			df = new SimpleDateFormat(aMask);
			returnValue = df.format(aDate);
		}

		return (returnValue);
	}

	/**
	 * This method generates a string representation of a date based on the
	 * System Property 'dateFormat' in the format you specify on input
	 * 
	 * @param aDate
	 *            A date to convert
	 * @return a string representation of the date
	 */
	public static final String convertDateToString(Date aDate) {
		return getDateTime(outDatePattern, aDate);
	}

	/**
	 * This method converts a String to a date using the datePattern
	 * 
	 * @param strDate
	 *            the date to convert (in format MM/dd/yyyy)
	 * @return a date object
	 * 
	 * @throws ParseException
	 */
	public static Date convertStringToDate(String strDate)
			throws ParseException {
		Date aDate = null;

		try {
			if (log.isDebugEnabled()) {
				log.debug("converting date with pattern: " + outDatePattern);
			}

			aDate = convertStringToDate(outDatePattern, strDate);
		} catch (ParseException pe) {
			log.error("Could not convert '" + strDate
					+ "' to a date, throwing exception");
			pe.printStackTrace();
			throw new ParseException(pe.getMessage(), pe.getErrorOffset());

		}

		return aDate;
	}

	public static String convertDate(Date value) {
		try {
			return formatDMY.format(value);
		} catch (Exception e) {
			return "?";
		}

	}

	public static Date stringYMD2Date(String textDate) {
		if (textDate == null || "".equals(textDate) || "null".equals(textDate))
			  return null;		
		try {
			Date date = formatYMD.parse(textDate);
		    return date;
		} catch (Exception e) {
			return null;
		}
	}
	
	
	public static Date mySqlStringToDate(String textDate) { //dtecter le format de la date, varie
		if (textDate == null || "".equals(textDate) || "null".equals(textDate))
			  return null;		
		try {
			Date date;
			String sep = textDate.substring(4,5);//"yyyy-MM-dd HH:MM:SS");
			if ("-".equals(sep))
			   date = formatYMDHMS1.parse(textDate);
			else
			   date =  formatYMDHMS2.parse(textDate);
		    return date;
		} catch (Exception e) {
			return null;
		}
	}
	
	public static Date stringDMY2Date(String textDate) {
		if (textDate == null || "".equals(textDate) || "null".equals(textDate))
		  return null;
		try {
			Date date = formatDMY.parse(textDate);
		    return date;
		} catch (Exception e) {
			return null;//stringYMD2Date("01/01/1901");
		}
	}
	
	
	public static Date toFrenchDate(String textDate) {  //Sat Jan 21 14:00:00 CET 2006
		if (textDate == null || "".equals(textDate) || "null".equals(textDate))
			  return null;		
		try {
			//englishPattern2.setLenient(true);
			Date date = englishPattern2.parse(textDate); //java.text.ParseException: Unparseable date: "Sat Jan 21 14:00:00 CET 2006"
		    return date;
		} catch (Exception e) {
			return null;//stringYMD2Date("1900-01-01");
		}
	}	
	
	public static String englishToFrenchDate(String textDate) { //Sat Jan 21 14:00:00 CET 2006 => "dd/MM/yyyy"; //simplifier
		String result = "";//textDate;
		try {
		    result = getDateTime(outDatePattern, toFrenchDate(textDate));
			if (result == null || "".equals(result)){ 
				if (textDate != null)
					result = textDate;
				else
				    result = "&nbsp;";
			}	
			return result;
		} catch (Exception e) {
			return "date-error";
		}
	}	
	
	/*
	
	public static String convertMysqlDate(Date date) { 
		try {
			return englishPattern2.format(date);
		} catch (Exception e) {
			return "";
		}
	}	
	*/
	
	public static String convertDateFormat(String textDate) { 
		String result = "";
		try {
		    result = getDateTime(outDatePattern, stringYMD2Date(textDate));
			if (result == null || "".equals(result)){ 
				if (textDate != null)
					result = textDate;
				else
				    result = "&nbsp;";
			}	
			return result;
		} catch (Exception e) {
			return "date-error";
		}
	}	
	
	public static String convertDateFormat(Date date) { //simplifier
		try {
			return getDateTime(outDatePattern, date);
		} catch (Exception e) {
			return "";
		}
	}		
}
